"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const parseUrl = require("parseurl");
const qs = require("qs");
const json_1 = require("./body/json");
exports.JsonBody = json_1.JsonBody;
const text_1 = require("./body/text");
exports.TextBody = text_1.TextBody;
const body_1 = require("./errors/body");
exports.ErrorWithBody = body_1.ErrorWithBody;
const status_1 = require("./errors/status");
exports.ErrorWithStatusCode = status_1.ErrorWithStatusCode;
const express_1 = require("./express");
exports.useExpress = express_1.useExpress;
const router_1 = require("./router");
exports.Methods = router_1.Methods;
exports.Router = router_1.Router;
const utils_1 = require("./utils");
exports.defaultErrorHandler = (ctx, error) => {
    ctx.statusCode =
        error instanceof status_1.ErrorWithStatusCode ? error.statusCode : 500;
    ctx.body =
        error instanceof body_1.ErrorWithBody ? error.body : new text_1.TextBody(error.message);
};
class Routex extends router_1.Router {
    constructor() {
        super(...arguments);
        this.errorHandler = exports.defaultErrorHandler;
        this.handler = (req, res) => __awaiter(this, void 0, void 0, function* () {
            // Parse query string and extract path
            const parsed = parseUrl(req);
            const query = parsed &&
                (utils_1.isString(parsed.query)
                    ? qs.parse(parsed.query)
                    : parsed.query);
            /* istanbul ignore next */
            const ctx = {
                data: {},
                path: (parsed && parsed.pathname) || "/",
                query: query || {},
                req,
                res
            };
            yield this.handle(ctx);
            if (ctx.body) {
                ctx.res.setHeader("Content-Length", ctx.body.contentLength);
                ctx.res.setHeader("Content-Type", ctx.body.contentType);
            }
            if (ctx.statusCode) {
                ctx.res.writeHead(ctx.statusCode);
            }
            if (ctx.body) {
                ctx.body.write(ctx.res);
            }
            ctx.res.end();
        });
    }
    listen(port, { hostname } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            if (port) {
                const originalPort = port;
                if (utils_1.isString(port)) {
                    port = parseInt(port, 10);
                }
                if (!Number.isInteger(port)) {
                    throw new Error(`Invalid port (${originalPort} is not integer).`);
                }
            }
            if (hostname && !utils_1.isString(hostname)) {
                throw new Error(`Invalid hostname (${hostname} is not a string)`);
            }
            const server = http.createServer(this.handler);
            yield new Promise(resolve => {
                server.listen(port, hostname, resolve);
            });
            const close = () => new Promise((resolve, reject) => {
                server.close(error => {
                    if (error) {
                        /* istanbul ignore next */
                        reject(error);
                    }
                    resolve();
                });
            });
            const address = server.address();
            return {
                close,
                port: 
                /* istanbul ignore next */
                address && !utils_1.isString(address) ? address.port : null,
                server
            };
        });
    }
}
exports.Routex = Routex;
//# sourceMappingURL=index.js.map