"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const status_1 = require("./errors/status");
const route_1 = require("./route");
const utils_1 = require("./utils");
var Methods;
(function (Methods) {
    Methods["GET"] = "get";
    Methods["POST"] = "post";
    Methods["DELETE"] = "delete";
    Methods["PUT"] = "put";
    Methods["PATCH"] = "patch";
})(Methods = exports.Methods || (exports.Methods = {}));
class Router {
    constructor() {
        this.routes = [];
        this.middlewares = [];
        this.child = (path, handler, options) => {
            const router = handler || new Router();
            this.routes.push(new route_1.Route(undefined, path, router, options));
            return router;
        };
        this.route = (method, path, handler, options) => {
            this.routes.push(new route_1.Route(utils_1.toLowerCases(method), path, handler, Object.assign({ exact: true }, options)));
            return this;
        };
        this.any = (path, handler, options) => {
            return this.route(undefined, path, handler, options);
        };
        this.middleware = (middleware) => {
            if (middleware) {
                this.middlewares.push(middleware);
            }
            return this;
        };
        this.handle = (ctx) => __awaiter(this, void 0, void 0, function* () {
            try {
                const middlewaresNext = [];
                for (const middleware of this.middlewares) {
                    const next = yield middleware(ctx);
                    if (next) {
                        middlewaresNext.push(next);
                    }
                }
                if (!ctx.req.method || !ctx.req.url) {
                    /* istanbul ignore next */
                    // Invalid HTTP request, should never happen
                    throw new status_1.ErrorWithStatusCode(400, "Invalid request");
                }
                if (!ctx.path || !ctx.path.startsWith("/")) {
                    ctx.path = "/" + (ctx.path || "");
                }
                const method = ctx.req.method.toLowerCase();
                const url = ctx.path;
                let route;
                let match;
                this.routes.some(testRoute => {
                    const matchesMethod = testRoute.method &&
                        testRoute.method !== method &&
                        !(Array.isArray(testRoute.method) && testRoute.method.includes(method));
                    if (matchesMethod) {
                        return false;
                    }
                    if (testRoute.regex) {
                        match = testRoute.regex.exec(url);
                        if (!match) {
                            return false;
                        }
                    }
                    if (match) {
                        ctx.path = ctx.path.replace(match[0], "");
                    }
                    route = testRoute;
                    return true;
                });
                if (route) {
                    yield route.handle(ctx);
                    for (const middelwareNext of middlewaresNext) {
                        yield middelwareNext();
                    }
                    return;
                }
                // 404
                throw new status_1.ErrorWithStatusCode(404, "Not Found");
            }
            catch (error) {
                if (this.errorHandler) {
                    return this.errorHandler(ctx, error);
                }
                throw error;
            }
        });
    }
}
exports.Router = Router;
// Attach method shortcuts such as router.get(...), etc
Object.values(Methods).forEach(method => {
    // @ts-ignore
    Router.prototype[method] = function (path, handler, options) {
        return this.route(method, path, handler, options);
    };
});
//# sourceMappingURL=router.js.map